# Generated by roxygen2: do not edit by hand

S3method("!=",fdata)
S3method("*",fdata)
S3method("+",fdata)
S3method("-",fdata)
S3method("/",fdata)
S3method("==",fdata)
S3method("[",fdata)
S3method("[",fdist)
S3method("[",ldata)
S3method("[",mfdata)
S3method("^",fdata)
S3method(Math,fdata)
S3method(Math,ldata)
S3method(Math,mfdata)
S3method(Ops,fdata)
S3method(Ops,ldata)
S3method(Ops,mfdata)
S3method(Summary,fdata)
S3method(Summary,ldata)
S3method(Summary,mfdata)
S3method(anyNA,fdata)
S3method(c,fdata)
S3method(c,ldata)
S3method(c,mdata)
S3method(dim,fdata)
S3method(influence,fregre.fd)
S3method(is.na,fdata)
S3method(length,fdata)
S3method(lines,fdata)
S3method(mean,fdata)
S3method(mean,ldata)
S3method(mean,mfdata)
S3method(na.fail,fdata)
S3method(na.omit,fdata)
S3method(names,ldata)
S3method(names,mfdata)
S3method(plot,bifd)
S3method(plot,depth)
S3method(plot,fdata)
S3method(plot,ldata)
S3method(plot,mdepth)
S3method(plot,mfdata)
S3method(plot,summary.lm)
S3method(predict,classif)
S3method(predict,classif.DD)
S3method(predict,fregre.fd)
S3method(predict,fregre.fr)
S3method(predict,fregre.gkam)
S3method(predict,fregre.glm)
S3method(predict,fregre.gls)
S3method(predict,fregre.gsam)
S3method(predict,fregre.igls)
S3method(predict,fregre.lm)
S3method(predict,fregre.plm)
S3method(print,classif)
S3method(print,fregre.fd)
S3method(print,fregre.gkam)
S3method(print,fregre.plm)
S3method(split,fdata)
S3method(subset,fdata)
S3method(subset,ldata)
S3method(subset,mfdata)
S3method(summary,basis.fdata)
S3method(summary,classif)
S3method(summary,fanova.RPm)
S3method(summary,fdata.comp)
S3method(summary,fregre.fd)
S3method(summary,fregre.gkam)
S3method(summary,fregre.lm)
export(AKer.cos)
export(AKer.epa)
export(AKer.norm)
export(AKer.quar)
export(AKer.tri)
export(AKer.unif)
export(Adot)
export(CV.S)
export(FDR)
export(Ftest.statistic)
export(GCCV.S)
export(GCV.S)
export(IKer.cos)
export(IKer.epa)
export(IKer.norm)
export(IKer.quar)
export(IKer.tri)
export(IKer.unif)
export(Ker.cos)
export(Ker.epa)
export(Ker.norm)
export(Ker.quar)
export(Ker.tri)
export(Ker.unif)
export(Kernel)
export(Kernel.asymmetric)
export(Kernel.integrate)
export(LMDC.regre)
export(LMDC.select)
export(MMD.test)
export(MMDA.test)
export(NCOL.fdata)
export(NCOL.ldata)
export(NCOL.mfdata)
export(NROW.fdata)
export(NROW.ldata)
export(NROW.mfdata)
export(P.penalty)
export(PCvM.statistic)
export(S.KNN)
export(S.LCR)
export(S.LLR)
export(S.LPR)
export(S.NW)
export(S.basis)
export(Var.e)
export(Var.y)
export(XYRP.test)
export(argvals)
export(argvals.equi)
export(bcdcor.dist)
export(cat2meas)
export(classif.DD)
export(classif.cv.glmnet)
export(classif.depth)
export(classif.gbm)
export(classif.gkam)
export(classif.glm)
export(classif.gsam)
export(classif.gsam.vs)
export(classif.kernel)
export(classif.kfold)
export(classif.knn)
export(classif.ksvm)
export(classif.lda)
export(classif.naiveBayes)
export(classif.nnet)
export(classif.np)
export(classif.qda)
export(classif.randomForest)
export(classif.rpart)
export(classif.svm)
export(colnames.fdata)
export(cond.F)
export(cond.mode)
export(cond.quantile)
export(count.na.fdata)
export(cov.test.fdata)
export(create.fdata.basis)
export(create.pc.basis)
export(create.pls.basis)
export(create.raw.fdata)
export(dcor.dist)
export(dcor.test)
export(dcor.xy)
export(depth.FM)
export(depth.FMp)
export(depth.FSD)
export(depth.KFSD)
export(depth.RP)
export(depth.RPD)
export(depth.RPp)
export(depth.RT)
export(depth.mode)
export(depth.modep)
export(dev.S)
export(dfv.statistic)
export(dfv.test)
export(dis.cos.cor)
export(fEqDistrib.test)
export(fanova.RPm)
export(fanova.hetero)
export(fanova.onefactor)
export(fdata)
export(fdata.bootstrap)
export(fdata.cen)
export(fdata.deriv)
export(fdata2basis)
export(fdata2fd)
export(fdata2pc)
export(fdata2pls)
export(flm.Ftest)
export(flm.test)
export(fmean.test.fdata)
export(fregre.basis)
export(fregre.basis.cv)
export(fregre.basis.fr)
export(fregre.bootstrap)
export(fregre.gkam)
export(fregre.glm)
export(fregre.glm.vs)
export(fregre.gls)
export(fregre.gsam)
export(fregre.gsam.vs)
export(fregre.igls)
export(fregre.lm)
export(fregre.np)
export(fregre.np.cv)
export(fregre.pc)
export(fregre.pc.cv)
export(fregre.plm)
export(fregre.pls)
export(fregre.pls.cv)
export(func.mean)
export(func.mean.formula)
export(func.med.FM)
export(func.med.RP)
export(func.med.RPD)
export(func.med.RT)
export(func.med.mode)
export(func.trim.FM)
export(func.trim.RP)
export(func.trim.RPD)
export(func.trim.RT)
export(func.trim.mode)
export(func.trimvar.FM)
export(func.trimvar.RP)
export(func.trimvar.RPD)
export(func.trimvar.RT)
export(func.trimvar.mode)
export(func.var)
export(gridfdata)
export(h.default)
export(influence_quan)
export(inprod.fdata)
export(int.simpson)
export(int.simpson2)
export(is.fdata)
export(is.ldata)
export(is.mfdata)
export(kmeans.center.ini)
export(kmeans.fd)
export(ldata)
export(ldata.cen)
export(mdepth.FM)
export(mdepth.FSD)
export(mdepth.HS)
export(mdepth.KFSD)
export(mdepth.LD)
export(mdepth.MhD)
export(mdepth.RP)
export(mdepth.SD)
export(mdepth.TD)
export(metric.DTW)
export(metric.TWED)
export(metric.WDTW)
export(metric.dist)
export(metric.hausdorff)
export(metric.kl)
export(metric.ldata)
export(metric.lp)
export(metric.mfdata)
export(mfdata)
export(mfdata.cen)
export(ncol.fdata)
export(ncol.ldata)
export(ncol.mfdata)
export(norm.fd)
export(norm.fdata)
export(nrow.fdata)
export(nrow.ldata)
export(nrow.mfdata)
export(ops.fda.usc)
export(optim.basis)
export(optim.np)
export(order.fdata)
export(outliers.depth.pond)
export(outliers.depth.trim)
export(outliers.lrt)
export(outliers.thres.lrt)
export(plot.fdata)
export(plot.lfdata)
export(pred.MAE)
export(pred.MSE)
export(pred.RMSE)
export(pred2meas)
export(pvalue.FDR)
export(r.ou)
export(rangeval)
export(rcombfdata)
export(rdir.pc)
export(rownames.fdata)
export(rp.flm.statistic)
export(rp.flm.test)
export(rproc2fdata)
export(rwild)
export(semimetric.basis)
export(semimetric.deriv)
export(semimetric.fourier)
export(semimetric.hshift)
export(semimetric.mplsr)
export(semimetric.pca)
export(tab2meas)
export(title.fdata)
export(trace.matrix)
export(unlist_fdata)
export(weights4class)
import(MASS)
import(fda)
import(grDevices)
import(knitr)
import(methods)
import(mgcv)
import(nlme)
import(parallel)
import(splines)
import(stats)
import(utils)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(foreach,getDoParName)
importFrom(foreach,getDoParRegistered)
importFrom(foreach,getDoParVersion)
importFrom(foreach,getDoParWorkers)
importFrom(foreach,registerDoSEQ)
importFrom(foreach,setDoPar)
importFrom(foreach,setDoSeq)
importFrom(graphics,abline)
importFrom(graphics,boxplot)
importFrom(graphics,contour)
importFrom(graphics,curve)
importFrom(graphics,filled.contour)
importFrom(graphics,image)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matlines)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,persp)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,rug)
importFrom(graphics,stars)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(iterators,icount)
importFrom(kSamples,ad.test)
useDynLib(fda.usc, .registration = TRUE)
